@@ -2,7 +2,7 @@
 /* clp.c - Complete source code for CLP.
  * This file is part of CLP, the command line parser package.
  *
- * Copyright (c) 1997-2010 Eddie Kohler, ekohler@gmail.com
+ * Copyright (c) 1997-2011 Eddie Kohler, ekohler@gmail.com
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -275,7 +275,7 @@ encode_utf8(char *s, int n, int c)
 }
 
 static int
-decode_utf8(const char *s, char **cp)
+decode_utf8(const char *s, const char **cp)
 {
     int c;
     if ((unsigned char) *s <= 0x7F)		/* 1 byte:  0x000000-0x00007F */
@@ -319,14 +319,14 @@ decode_utf8(const char *s, char **cp)
 	    /* nothing */;
     }
     if (cp)
-	*cp = (char *) s;
+	*cp = s;
     return c;
 }
 
 static int
 utf8_charlen(const char *s)
 {
-    char *sout;
+    const char *sout;
     (void) decode_utf8(s, &sout);
     return sout - s;
 }
@@ -364,7 +364,7 @@ long_as_short(const Clp_Internal *cli, const Clp_Option *o,
     if ((cli->long1pos || cli->long1neg) && io->ilong) {
 	const char *name = o->long_name + io->ilongoff;
 	if (cli->utf8) {
-	    int c = decode_utf8(name, (char **) &name);
+	    int c = decode_utf8(name, &name);
 	    if (!*name && c && c != U_REPLACEMENT)
 		return c;
 	} else if (name[0] && !name[1])
@@ -476,6 +476,7 @@ Clp_NewParser(int argc, const char * const *argv, int nopt, const Clp_Option *op
     if (!clp || !cli || !iopt || !cli->valtype)
 	goto failed;
 
+    clp->opt = -1;
     clp->negated = 0;
     clp->have_val = 0;
     clp->vstr = 0;
@@ -1073,22 +1074,22 @@ parse_string(Clp_Parser *clp, const char *arg, int complain, void *user_data)
 static int
 parse_int(Clp_Parser *clp, const char *arg, int complain, void *user_data)
 {
-    char *val;
+    const char *val;
     if (*arg == 0 || isspace((unsigned char) *arg)
 	|| (user_data != 0 && *arg == '-'))
-	val = (char *) arg;
+	val = arg;
     else if (user_data != 0) {	/* unsigned */
 #if HAVE_STRTOUL
-	clp->val.u = strtoul(arg, &val, 0);
+	clp->val.u = strtoul(arg, (char **) &val, 0);
 #else
 	/* don't bother really trying to do it right */
 	if (arg[0] == '-')
-	    val = (char *) arg;
+	    val = arg;
 	else
-	    clp->val.u = strtol(arg, &val, 0);
+	    clp->val.u = strtol(arg, (char **) &val, 0);
 #endif
     } else
-	clp->val.i = strtol(arg, &val, 0);
+	clp->val.i = strtol(arg, (char **) &val, 0);
     if (*arg != 0 && *val == 0)
 	return 1;
     else if (complain) {
@@ -1103,12 +1104,12 @@ parse_int(Clp_Parser *clp, const char *arg, int complain, void *user_data)
 static int
 parse_double(Clp_Parser *clp, const char *arg, int complain, void *user_data)
 {
-    char *val;
+    const char *val;
     (void)user_data;
     if (*arg == 0 || isspace((unsigned char) *arg))
-	val = (char *) arg;
+	val = arg;
     else
-	clp->val.d = strtod(arg, &val);
+	clp->val.d = strtod(arg, (char **) &val);
     if (*arg != 0 && *val == 0)
 	return 1;
     else if (complain)
@@ -1521,7 +1522,7 @@ get_oclass(Clp_Parser *clp, const char *text, int *ocharskip)
 {
     int c;
     if (clp->internal->utf8) {
-	char *s;
+	const char *s;
 	c = decode_utf8(text, &s);
 	*ocharskip = s - text;
     } else {
@@ -1820,7 +1821,8 @@ Clp_Next(Clp_Parser *clp)
     /* Get the next argument or option */
     if (!next_argument(clp, cli->option_processing ? 0 : 2)) {
 	clp->val.s = clp->vstr;
-	return clp->have_val ? Clp_NotOption : Clp_Done;
+	clp->opt = clp->have_val ? Clp_NotOption : Clp_Done;
+	return clp->opt;
     }
 
     clp->negated = cli->whole_negated;
@@ -1858,7 +1860,7 @@ Clp_Next(Clp_Parser *clp)
 	    Clp_OptionError(clp, "unrecognized option %<%s%s%>",
 			    cli->option_chars, cli->xtext);
 
-	return Clp_BadOption;
+	return (clp->opt = Clp_BadOption);
     }
 
     /* Set the current option */
@@ -1871,19 +1873,20 @@ Clp_Next(Clp_Parser *clp)
 	|| (!cli->iopt[optno].imandatory && !cli->iopt[optno].ioptional)) {
 	if (clp->have_val) {
 	    Clp_OptionError(clp, "%<%O%> can%,t take an argument");
-	    return Clp_BadOption;
+	    clp->opt = Clp_BadOption;
 	} else
-	    return cli->opt[optno].option_id;
+	    clp->opt = cli->opt[optno].option_id;
+	return clp->opt;
     }
 
     /* Get an argument if we need one, or if it's optional */
     /* Sanity-check the argument type. */
     opt = &cli->opt[optno];
     if (opt->val_type <= 0)
-	return Clp_Error;
+	return (clp->opt = Clp_Error);
     vtpos = val_type_binsearch(cli, opt->val_type);
     if (vtpos == cli->nvaltype || cli->valtype[vtpos].val_type != opt->val_type)
-	return Clp_Error;
+	return (clp->opt = Clp_Error);
 
     /* complain == 1 only if the argument was explicitly given,
        or it is mandatory. */
@@ -1903,7 +1906,7 @@ Clp_Next(Clp_Parser *clp)
 		Clp_OptionError(clp, "%<%O%> requires a non-option argument");
 	    else
 		Clp_OptionError(clp, "%<%O%> requires an argument");
-	    return Clp_BadOption;
+	    return (clp->opt = Clp_BadOption);
 	}
 
     } else if (cli->is_short && !clp->have_val
@@ -1919,13 +1922,13 @@ Clp_Next(Clp_Parser *clp)
 	    /* parser failed */
 	    clp->have_val = 0;
 	    if (cli->iopt[optno].imandatory)
-		return Clp_BadOption;
+		return (clp->opt = Clp_BadOption);
 	    else
 		Clp_RestoreParser(clp, &clpsave);
 	}
     }
 
-    return opt->option_id;
+    return (clp->opt = opt->option_id);
 }
 
 
