@@ -1,6 +1,6 @@
 /* -*- c-basic-offset: 2 -*- */
 /* gifsicle.c - gifsicle's main loop.
-   Copyright (C) 1997-2010 Eddie Kohler, ekohler@cs.ucla.edu
+   Copyright (C) 1997-2011 Eddie Kohler, ekohler@cs.ucla.edu
    This file is part of gifsicle.
 
    Gifsicle is free software. It is distributed under the GNU Public License,
@@ -47,15 +47,8 @@ int warn_local_colormaps = 1;
 static Gt_ColorTransform *input_transforms;
 static Gt_ColorTransform *output_transforms;
 
-#define BLANK_MODE	0
-#define MERGING		1
-#define BATCHING	2
-#define EXPLODING	3
-#define INFOING		4
-#define DELETING	5
-#define INSERTING	6
-static int mode = BLANK_MODE;
-static int nested_mode = 0;
+int mode = BLANK_MODE;
+int nested_mode = 0;
 
 static int infoing = 0;
 int verbosing = 0;
@@ -183,6 +176,9 @@ static const char *output_option_types[] = {
 #define CONSERVE_MEMORY_OPT	361
 #define MULTIFILE_OPT		362
 #define NEXTFILE_OPT		363
+#define RESIZE_FIT_OPT		364
+#define RESIZE_FIT_WIDTH_OPT	365
+#define RESIZE_FIT_HEIGHT_OPT	366
 
 #define LOOP_TYPE		(Clp_ValFirstUser)
 #define DISPOSAL_TYPE		(Clp_ValFirstUser + 1)
@@ -262,6 +258,11 @@ const Clp_Option options[] = {
   { "resiz", 0, RESIZE_OPT, DIMENSIONS_TYPE, Clp_Negate },
   { "resi", 0, RESIZE_OPT, DIMENSIONS_TYPE, Clp_Negate },
   { "res", 0, RESIZE_OPT, DIMENSIONS_TYPE, Clp_Negate },
+  { "resize-fit", 0, RESIZE_FIT_OPT, DIMENSIONS_TYPE, Clp_Negate },
+  { "resize-fit-width", 0, RESIZE_FIT_WIDTH_OPT, Clp_ValUnsigned, Clp_Negate },
+  { "resize-fit-height", 0, RESIZE_FIT_HEIGHT_OPT, Clp_ValUnsigned, Clp_Negate },
+  { "resize-fi", 0, RESIZE_FIT_OPT, DIMENSIONS_TYPE, Clp_Negate },
+  { "resize-f", 0, RESIZE_FIT_OPT, DIMENSIONS_TYPE, Clp_Negate },
   { "rotate-90", 0, ROTATE_90_OPT, 0, 0 },
   { "rotate-180", 0, ROTATE_180_OPT, 0, 0 },
   { "rotate-270", 0, ROTATE_270_OPT, 0, 0 },
@@ -428,14 +429,19 @@ show_frame(int imagenumber, int usename)
 static int gifread_error_count;
 
 static void
-gifread_error(const char *message, int which_image, void *thunk)
+gifread_error(int is_error, const char *message, int which_image, void *thunk)
 {
+  static int last_is_error = 0;
   static int last_which_image = 0;
   static char last_message[256];
   static int different_error_count = 0;
   static int same_error_count = 0;
   const char *filename = (const char *)thunk;
 
+  /* ignore warnings if "no_warning" */
+  if (no_warnings && is_error == 0)
+    return;
+
   if (gifread_error_count == 0) {
     last_which_image = -1;
     last_message[0] = 0;
@@ -446,30 +452,28 @@ gifread_error(const char *message, int which_image, void *thunk)
   if (last_message[0] && different_error_count <= 10
       && (last_which_image != which_image || message == 0
 	  || strcmp(message, last_message) != 0)) {
+    const char *etype = last_is_error ? "error" : "warning";
+    error(0, "While reading '%s' frame #%d:", filename, last_which_image);
     if (same_error_count == 1)
-      error("  %s", last_message);
+      error(0, "  %s: %s", etype, last_message);
     else if (same_error_count > 0)
-      error("  %s (%d times)", last_message, same_error_count);
+      error(0, "  %s: %s (%d times)", etype, last_message, same_error_count);
     same_error_count = 0;
     last_message[0] = 0;
   }
 
-  if (last_message[0] == 0)
-    different_error_count++;
-
-  same_error_count++;
-  if (message)
+  if (message) {
+    if (last_message[0] == 0)
+      different_error_count++;
+    same_error_count++;
     strcpy(last_message, message);
-  else
-    last_message[0] = 0;
-  if (last_which_image != which_image && different_error_count <= 10
-      && message) {
-    error("Error while reading '%s' frame #%d:", filename, which_image);
     last_which_image = which_image;
-  }
+    last_is_error = is_error;
+  } else
+    last_message[0] = 0;
 
   if (different_error_count == 11 && message) {
-    error("(more errors while reading '%s')", filename);
+    error(0, "(more errors while reading '%s')", filename);
     different_error_count++;
   }
 }
@@ -492,7 +496,7 @@ open_giffile(const char *name)
 #ifndef OUTPUT_GIF_TO_TERMINAL
     extern int isatty(int);
     if (isatty(fileno(stdin))) {
-      error("<stdin>: is a terminal");
+      error(0, "<stdin>: is a terminal");
       return NULL;
     }
 #endif
@@ -520,7 +524,7 @@ open_giffile(const char *name)
     stored_files = sf;
     strcpy(sf->name, name);
   } else if (!f)
-    error("%s: %s", name, strerror(errno));
+    error(0, "%s: %s", name, strerror(errno));
 
   return f;
 }
@@ -599,9 +603,9 @@ input_stream(const char *name)
   i = getc(f);
   if (i == EOF) {
     if (!(gif_read_flags & GIF_READ_TRAILING_GARBAGE_OK))
-      error("%s: empty file", name);
+      error(0, "%s: empty file", name);
     else if (nextfile)
-      error("%s: no more images in file", name);
+      error(0, "%s: no more images in file", name);
     close_giffile(f, 1);
     return;
   }
@@ -614,13 +618,13 @@ input_stream(const char *name)
   gifread_error_count = 0;
   gfs = Gif_FullReadFile(f, gif_read_flags | GIF_READ_COMPRESSED,
 			 gifread_error, (void *)name);
-  gifread_error(0, -1, (void *)name); /* print out last error message */
+  gifread_error(-1, 0, -1, (void *)name); /* print out last error message */
 
   if (!gfs || (Gif_ImageCount(gfs) == 0 && gfs->errors > 0)) {
     if (componentno == 1)
-      error("%s: not a GIF image", name);
+      error(0, "%s: file not in GIF format", name);
     else
-      error("%s: trailing garbage ignored", main_name);
+      error(0, "%s: trailing garbage ignored", main_name);
     Gif_DeleteStream(gfs);
     if (verbosing)
       verbose_close('>');
@@ -696,10 +700,10 @@ input_stream(const char *name)
   if (unoptimizing)
     if (!Gif_FullUnoptimize(gfs, GIF_UNOPTIMIZE_SIMPLEST_DISPOSAL)) {
       static int context = 0;
-      warning("'%s' is too complex to unoptimize", name);
+      warning(1, "GIF too complex to unoptimize", name);
       if (!context) {
-	warncontext("(The reason was local color tables or complex transparency.");
-	warncontext("Try running the GIF through 'gifsicle --colors=255' first.)");
+	warncontext(1, "(The reason was local color tables or complex transparency.");
+	warncontext(1, "Try running the GIF through 'gifsicle --colors=255' first.)");
       }
       context = 1;
     }
@@ -797,7 +801,7 @@ do_colormap_change(Gif_Stream *gfs)
 	  any_locals = 1;
       hist = histogram(gfs, &nhist);
       if (nhist <= active_output_data.colormap_size && !any_locals) {
-	warning("trivial adaptive palette (only %d colors in source)", nhist);
+	warning(1, "trivial adaptive palette (only %d colors in source)", nhist);
 	return;
       }
     }
@@ -845,7 +849,7 @@ write_stream(const char *output_name, Gif_Stream *gfs)
 #ifndef OUTPUT_GIF_TO_TERMINAL
     extern int isatty(int);
     if (isatty(fileno(stdout))) {
-      error("<stdout>: is a terminal");
+      error(0, "<stdout>: is a terminal");
       return;
     }
 #endif
@@ -865,7 +869,7 @@ write_stream(const char *output_name, Gif_Stream *gfs)
     fclose(f);
     any_output_successful = 1;
   } else
-    error("%s: %s", output_name, strerror(errno));
+    error(0, "%s: %s", output_name, strerror(errno));
 }
 
 static void
@@ -894,12 +898,15 @@ merge_and_write_frames(const char *outfile, int f1, int f2)
 			     compress_immediately, &huge_stream);
 
   if (out) {
-    if (active_output_data.scaling == 1)
+    if (active_output_data.scaling == GT_SCALING_RESIZE)
       resize_stream(out, active_output_data.resize_width,
-		    active_output_data.resize_height);
-    else if (active_output_data.scaling == 2)
+		    active_output_data.resize_height, 0);
+    else if (active_output_data.scaling == GT_SCALING_SCALE)
       resize_stream(out, active_output_data.scale_x * out->screen_width,
-		    active_output_data.scale_y * out->screen_height);
+		    active_output_data.scale_y * out->screen_height, 0);
+    else if (active_output_data.scaling == GT_SCALING_RESIZE_FIT)
+      resize_stream(out, active_output_data.resize_width,
+		    active_output_data.resize_height, 1);
     if (colormap_change)
       do_colormap_change(out);
     if (output_transforms)
@@ -928,7 +935,7 @@ output_information(const char *outfile)
   else {
     f = fopen(outfile, "w");
     if (!f) {
-      error("%s: %s", outfile, strerror(errno));
+      error(0, "%s: %s", outfile, strerror(errno));
       return;
     }
   }
@@ -1130,7 +1137,7 @@ initialize_def_frame(void)
   def_output_data.colormap_dither = 0;
 
   def_output_data.optimizing = 0;
-  def_output_data.scaling = 0;
+  def_output_data.scaling = GT_SCALING_NONE;
 
   def_output_data.conserve_memory = 0;
 
@@ -1192,11 +1199,11 @@ static void
 redundant_option_warning(const char *option_type)
 {
   static int context = 0;
-  warning("redundant %s option", option_type);
+  warning(0, "redundant %s option", option_type);
   if (!context) {
-    warncontext("(The %s option was overridden by another %s option",
+    warncontext(0, "(The %s option was overridden by another %s option",
 		option_type, option_type);
-    warncontext("before it had any effect.)");
+    warncontext(0, "before it had any effect.)");
   }
   context = 1;
 }
@@ -1209,9 +1216,9 @@ print_useless_options(const char *type_name, int value, const char *names[])
   if (!value) return;
   for (i = 0; i < 32; i++)
     if (CHANGED(value, i)) {
-      warning("useless %s-related %s option", names[i], type_name);
+      warning(0, "useless %s-related %s option", names[i], type_name);
       if (!explanation_printed)
-	warncontext("(It didn't affect any %s.)", type_name);
+	warncontext(0, "(It didn't affect any %s.)", type_name);
       explanation_printed = 1;
     }
 }
@@ -1568,7 +1575,7 @@ main(int argc, char *argv[])
       if (clp->negated)
 	def_frame.disposal = GIF_DISPOSAL_NONE;
       else if (clp->val.i < 0 || clp->val.i > 7)
-	error("disposal must be between 0 and 7");
+	error(0, "disposal must be between 0 and 7");
       else
 	def_frame.disposal = clp->val.i;
       break;
@@ -1619,7 +1626,7 @@ main(int argc, char *argv[])
 	 input_transforms = delete_color_transforms
 	   (input_transforms, &color_change_transformer);
        else if (parsed_color2.haspixel)
-	 error("COLOR2 must be in RGB format in '--change-color COLOR1 COLOR2'");
+	 error(0, "COLOR2 must be in RGB format in '--change-color COLOR1 COLOR2'");
        else
 	 input_transforms = append_color_change
 	   (input_transforms, parsed_color, parsed_color2);
@@ -1644,7 +1651,7 @@ main(int argc, char *argv[])
 	def_output_data.colormap_size = clp->val.i;
 	if (def_output_data.colormap_size < 2
 	    || def_output_data.colormap_size > 256) {
-	  Clp_OptionError(clp, "argument to '%O' must be between 2 and 256");
+	  Clp_OptionError(clp, "argument to %O must be between 2 and 256");
 	  def_output_data.colormap_size = 0;
 	}
       }
@@ -1669,55 +1676,50 @@ main(int argc, char *argv[])
       def_output_data.colormap_dither = !clp->negated;
       break;
 
-     case RESIZE_OPT:
+    case RESIZE_OPT:
+    case RESIZE_FIT_OPT:
       MARK_CH(output, CH_RESIZE);
       if (clp->negated)
-	def_output_data.scaling = 0;
+	def_output_data.scaling = GT_SCALING_NONE;
       else if (dimensions_x <= 0 && dimensions_y <= 0) {
-	error("one of W and H must be positive in '--resize WxH'");
-	def_output_data.scaling = 0;
+	error(0, "one of W and H must be positive in '%s WxH'", Clp_CurOptionName(clp));
+	def_output_data.scaling = GT_SCALING_NONE;
       } else {
-	def_output_data.scaling = 1; /* use resize dimensions */
+	def_output_data.scaling = (opt == RESIZE_FIT_OPT ? GT_SCALING_RESIZE_FIT : GT_SCALING_RESIZE);
 	def_output_data.resize_width = dimensions_x;
 	def_output_data.resize_height = dimensions_y;
       }
       break;
 
-     case RESIZE_WIDTH_OPT:
+    case RESIZE_WIDTH_OPT:
+    case RESIZE_HEIGHT_OPT:
+    case RESIZE_FIT_WIDTH_OPT:
+    case RESIZE_FIT_HEIGHT_OPT:
       MARK_CH(output, CH_RESIZE);
       if (clp->negated)
-	def_output_data.scaling = 0;
+	def_output_data.scaling = GT_SCALING_NONE;
       else if (clp->val.u == 0) {
-	error("'--resize-width' argument must be positive");
-	def_output_data.scaling = 0;
+	error(0, "%s argument must be positive", Clp_CurOptionName(clp));
+	def_output_data.scaling = GT_SCALING_NONE;
       } else {
-	def_output_data.scaling = 1; /* use resize dimensions */
-	def_output_data.resize_width = clp->val.u;
-	def_output_data.resize_height = 0;
-      }
-      break;
-
-     case RESIZE_HEIGHT_OPT:
-      MARK_CH(output, CH_RESIZE);
-      if (clp->negated)
-	def_output_data.scaling = 0;
-      else if (clp->val.u == 0) {
-	error("'--resize-height' argument must be positive");
-	def_output_data.scaling = 0;
-      } else {
-	def_output_data.scaling = 1; /* use resize dimensions */
-	def_output_data.resize_width = 0;
-	def_output_data.resize_height = clp->val.u;
+	unsigned dimen[2] = {0, 0};
+	dimen[(opt == RESIZE_HEIGHT_OPT || opt == RESIZE_FIT_HEIGHT_OPT)] = clp->val.u;
+	if (opt == RESIZE_FIT_WIDTH_OPT || opt == RESIZE_FIT_HEIGHT_OPT)
+	  def_output_data.scaling = GT_SCALING_RESIZE_FIT;
+	else
+	  def_output_data.scaling = GT_SCALING_RESIZE;
+	def_output_data.resize_width = dimen[0];
+	def_output_data.resize_height = dimen[1];
       }
       break;
 
      case SCALE_OPT:
       MARK_CH(output, CH_RESIZE);
       if (clp->negated)
-	def_output_data.scaling = 0;
+	def_output_data.scaling = GT_SCALING_NONE;
       else if (parsed_scale_factor_x <= 0 || parsed_scale_factor_y <= 0) {
-	error("'--scale' X and Y factors must be positive");
-	def_output_data.scaling = 0;
+	error(0, "%s X and Y factors must be positive", Clp_CurOptionName(clp));
+	def_output_data.scaling = GT_SCALING_NONE;
       } else {
 	def_output_data.scaling = 2; /* use scale factor */
 	def_output_data.scale_x = parsed_scale_factor_x;
@@ -1764,7 +1766,7 @@ main(int argc, char *argv[])
 #else
       printf("LCDF Gifsicle %s\n", VERSION);
 #endif
-      printf("Copyright (C) 1997-2010 Eddie Kohler\n\
+      printf("Copyright (C) 1997-2011 Eddie Kohler\n\
 This is free software; see the source for copying conditions.\n\
 There is NO warranty, not even for merchantability or fitness for a\n\
 particular purpose.\n");
@@ -1829,5 +1831,6 @@ particular purpose.\n");
 #ifdef DMALLOC
   dmalloc_report();
 #endif
+  Clp_DeleteParser(clp);
   return (error_count ? EXIT_ERR : EXIT_OK);
 }
