@@ -119,6 +119,11 @@ typedef struct {
 
 } Gt_OutputData;
 
+#define GT_SCALING_NONE		0
+#define GT_SCALING_RESIZE	1
+#define GT_SCALING_SCALE	2
+#define GT_SCALING_RESIZE_FIT	3
+
 
 /*****
  * helper
@@ -205,7 +210,7 @@ int	crop_image(Gif_Image *, Gt_Crop *, int preserve_total_crop);
 void	flip_image(Gif_Image *, int scr_width, int scr_height, int is_vert);
 void	rotate_image(Gif_Image *, int scr_width, int scr_height, int rotation);
 void	scale_image(Gif_Stream *, Gif_Image *, double xfactor, double yfactor);
-void	resize_stream(Gif_Stream *, int new_width, int new_height);
+void	resize_stream(Gif_Stream *, int new_width, int new_height, int fit);
 
 /*****
  * quantization
