@@ -1,5 +1,5 @@
 /* optimize.c - Functions to optimize animated GIFs.
-   Copyright (C) 1997-2008 Eddie Kohler, ekohler@gmail.com
+   Copyright (C) 1997-2011 Eddie Kohler, ekohler@gmail.com
    This file is part of gifsicle.
 
    Gifsicle is free software. It is distributed under the GNU Public License,
@@ -352,8 +352,10 @@ find_difference_bounds(Gif_OptData *bounds, Gif_Image *gfi, Gif_Image *last)
   }
 
   /* 19.Aug.1999 - handle case when there's no difference between frames */
-  if (tp > bt)
-    tp = bt = lf = rt = 0;
+  if (tp > bt) {
+    tp = bt = gfi->top;
+    lf = rt = gfi->left;
+  }
 
   bounds->left = lf;
   bounds->top = tp;
@@ -375,12 +377,12 @@ expand_difference_bounds(Gif_OptData *bounds, Gif_Image *this_bounds)
 {
   int x, y, expanded = 0;
 
-  int lf = bounds->left, tp = bounds->top;
-  int rt = lf + bounds->width - 1, bt = tp + bounds->height - 1;
+  int lf = bounds->left, tp = bounds->top,
+      rt = lf + bounds->width - 1, bt = tp + bounds->height - 1;
 
   Gif_OptBounds ob = safe_bounds(this_bounds);
-  int tlf = ob.left, ttp = ob.top, trt = ob.left + ob.width - 1,
-      tbt = ob.top + ob.height - 1;
+  int tlf = ob.left, ttp = ob.top,
+      trt = ob.left + ob.width - 1, tbt = ob.top + ob.height - 1;
 
   if (lf > rt || tp > bt)
     lf = 0, tp = 0, rt = screen_width - 1, bt = screen_height - 1;
@@ -805,6 +807,20 @@ create_out_global_map(Gif_Stream *gfs)
  * CREATE COLOR MAPPING FOR A PARTICULAR IMAGE
  **/
 
+/* sort_colormap_permutation_rgb: for canonicalizing local colormaps by
+   arranging them in RGB order */
+
+static int
+colormap_rgb_permutation_sorter(const void *v1, const void *v2)
+{
+  const Gif_Color *col1 = (const Gif_Color *)v1;
+  const Gif_Color *col2 = (const Gif_Color *)v2;
+  int value1 = (col1->red << 16) | (col1->green << 8) | col1->blue;
+  int value2 = (col2->red << 16) | (col2->green << 8) | col2->blue;
+  return value1 - value2;
+}
+
+
 /* prepare_colormap_map: Create and return an array of bytes mapping from
    global pixel values to pixel values for this image. It may add colormap
    cells to 'into'; if there isn't enough room in 'into', it will return 0. It
@@ -849,6 +865,7 @@ prepare_colormap_map(Gif_Image *gfi, Gif_Colormap *into, uint8_t *need)
 	goto error;
       val = ncol;
       col[val] = all_col[i];
+      col[val].pixel = i;
       ncol++;
     }
 
@@ -856,6 +873,12 @@ prepare_colormap_map(Gif_Image *gfi, Gif_Colormap *into, uint8_t *need)
     into_used[val] = 1;
   }
 
+  if (!is_global) {
+    qsort(col, ncol, sizeof(Gif_Color), colormap_rgb_permutation_sorter);
+    for (i = 0; i < ncol; ++i)
+      map[col[i].pixel] = i;
+  }
+
   /* now check for transparency */
   gfi->transparent = -1;
   if (need[TRANSP]) {
@@ -904,20 +927,6 @@ prepare_colormap_map(Gif_Image *gfi, Gif_Colormap *into, uint8_t *need)
 }
 
 
-/* sort_colormap_permutation_rgb: for canonicalizing local colormaps by
-   arranging them in RGB order */
-
-static int
-colormap_rgb_permutation_sorter(const void *v1, const void *v2)
-{
-  const Gif_Color *col1 = (const Gif_Color *)v1;
-  const Gif_Color *col2 = (const Gif_Color *)v2;
-  int value1 = (col1->red << 16) | (col1->green << 8) | col1->blue;
-  int value2 = (col2->red << 16) | (col2->green << 8) | col2->blue;
-  return value1 - value2;
-}
-
-
 /* prepare_colormap: make a colormap up from the image data by fitting any
    used colors into a colormap. Returns a map from global color index to index
    in this image's colormap. May set a local colormap on 'gfi'. */
@@ -934,33 +943,8 @@ prepare_colormap(Gif_Image *gfi, uint8_t *need)
 
   if (!map) {
     /* that didn't work; add a local colormap. */
-    uint8_t permutation[256];
-    Gif_Color *local_col;
-    int i;
-
     gfi->local = Gif_NewFullColormap(0, 256);
     map = prepare_colormap_map(gfi, gfi->local, need);
-
-    /* The global colormap has already been canonicalized; we should
-       canonicalize the local colormaps as well. Do that here */
-    local_col = gfi->local->col;
-    for (i = 0; i < gfi->local->ncol; i++)
-      local_col[i].pixel = i;
-
-    qsort(local_col, gfi->local->ncol, sizeof(Gif_Color),
-	  colormap_rgb_permutation_sorter);
-
-    for (i = 0; i < gfi->local->ncol; i++)
-      permutation[local_col[i].pixel] = i;
-    /* 1.Aug.1999 - we might not have added space for gfi->transparent */
-    if (gfi->transparent >= gfi->local->ncol)
-      permutation[gfi->transparent] = gfi->transparent;
-
-    for (i = 0; i < all_colormap->ncol; i++)
-      map[i] = permutation[map[i]];
-
-    if (gfi->transparent >= 0)
-      gfi->transparent = map[TRANSP];
   }
 
   return map;
@@ -989,23 +973,47 @@ simple_frame_data(Gif_Image *gfi, uint8_t *map)
 }
 
 
+static void
+try_new_compression(Gif_Stream *gfs, Gif_Image *gfi, int write_flags)
+{
+    uint8_t *old_compressed = gfi->compressed;
+    void (*old_free_compressed)(void *) = gfi->free_compressed;
+    uint32_t old_compressed_len = gfi->compressed_len;
+    gfi->compressed = 0;	/* prevent freeing old_compressed */
+
+    Gif_FullCompressImage(gfs, gfi, write_flags);
+    if (gfi->compressed_len > old_compressed_len) {
+	Gif_ReleaseCompressedImage(gfi);
+	gfi->compressed = old_compressed;
+	gfi->free_compressed = old_free_compressed;
+	gfi->compressed_len = old_compressed_len;
+    } else
+	(*old_free_compressed)(old_compressed);
+}
+
+
 /* transp_frame_data: copy the frame data into the actual image, using
    transparency occasionally according to a heuristic described below */
 
 static void
-transp_frame_data(Gif_Stream *gfs, Gif_Image *gfi, uint8_t *map)
+transp_frame_data(Gif_Stream *gfs, Gif_Image *gfi, uint8_t *map,
+		  int optimize_level)
 {
   Gif_OptBounds ob = safe_bounds(gfi);
   int x, y, transparent = gfi->transparent;
   uint16_t *last = 0;
   uint16_t *cur = 0;
   uint8_t *data, *swit;
+  uint8_t *t2_data = 0, *last_for_t2;
   int transparentizing;
+  int write_flags = gif_write_flags;
+  if (optimize_level >= 3)
+      write_flags |= GIF_WRITE_OPTIMIZE;
 
   /* First, try w/o transparency. Compare this to the result using
      transparency and pick the better of the two. */
   simple_frame_data(gfi, map);
-  Gif_FullCompressImage(gfs, gfi, gif_write_flags);
+  Gif_FullCompressImage(gfs, gfi, write_flags);
 
   /* Actually copy data to frame.
 
@@ -1037,13 +1045,17 @@ transp_frame_data(Gif_Stream *gfs, Gif_Image *gfi, uint8_t *map)
      This simple heuristic does a little better than Gifwizard's (6/97)
      on some images, but does *worse than nothing at all* on others.
 
-     However, it DOES do better than the complicated, greedy algorithm I
-     commented out above; and now we pick either the transparency-optimized
-     version or the normal version, whichever compresses smaller, for the best
-     of both worlds. (9/98) */
+     However, it DOES do better than the complicated, greedy algorithm that
+     preceded it; and now we pick either the transparency-optimized version or
+     the normal version, whichever compresses smaller, for the best of both
+     worlds. (9/98)
+
+     On several images, making SINGLE color runs transparent wins over the
+     previous heuristic, so try both at optimize level 3 or above (the cost is
+     ~30%). (2/11) */
 
     x = y = 0;
-    data = swit = gfi->image_data;
+    data = swit = last_for_t2 = gfi->image_data;
     transparentizing = 0;
     goto calculate_pointers;
 
@@ -1066,6 +1078,15 @@ transp_frame_data(Gif_Stream *gfs, Gif_Image *gfi, uint8_t *map)
 		    transparentizing = 1;
 		    memset(swit, transparent, data - swit);
 		} else {
+		    if (data > swit && *data != *swit && optimize_level > 2) {
+			if (!t2_data)
+			    t2_data = Gif_NewArray(uint8_t, ob.width * ob.height);
+			memcpy(t2_data + (last_for_t2 - gfi->image_data),
+			       last_for_t2, swit - last_for_t2);
+			memset(t2_data + (swit - gfi->image_data),
+			       transparent, data - swit);
+			last_for_t2 = data;
+		    }
 		    if (*cur != *last)
 			swit = data + 1;
 		    else if (*swit != *data)
@@ -1084,23 +1105,19 @@ transp_frame_data(Gif_Stream *gfs, Gif_Image *gfi, uint8_t *map)
 	}
     }
 
+    if (t2_data)
+	memcpy(t2_data + (last_for_t2 - gfi->image_data),
+	       last_for_t2, data - last_for_t2);
 
-  /* Now, try compressed transparent version and pick the better of the two. */
-  {
-    uint8_t *old_compressed = gfi->compressed;
-    void (*old_free_compressed)(void *) = gfi->free_compressed;
-    uint32_t old_compressed_len = gfi->compressed_len;
-    gfi->compressed = 0;	/* prevent freeing old_compressed */
-    Gif_FullCompressImage(gfs, gfi, gif_write_flags);
-    if (gfi->compressed_len > old_compressed_len) {
-      Gif_ReleaseCompressedImage(gfi);
-      gfi->compressed = old_compressed;
-      gfi->free_compressed = old_free_compressed;
-      gfi->compressed_len = old_compressed_len;
-    } else
-      (*old_free_compressed)(old_compressed);
+
+    /* Now, try compressed transparent version(s) and pick the better of the
+       two (or three). */
+    try_new_compression(gfs, gfi, write_flags);
+    if (t2_data) {
+	Gif_SetUncompressedImage(gfi, t2_data, Gif_DeleteArrayFunc, 0);
+	try_new_compression(gfs, gfi, write_flags);
+    }
     Gif_ReleaseUncompressedImage(gfi);
-  }
 }
 
 
@@ -1167,7 +1184,7 @@ create_new_image_data(Gif_Stream *gfs, int optimize_level)
 
       /* don't use transparency on first frame */
       if (optimize_level > 1 && image_index > 0 && cur_gfi->transparent >= 0)
-	transp_frame_data(gfs, cur_gfi, map);
+	transp_frame_data(gfs, cur_gfi, map, optimize_level);
       else
 	simple_frame_data(cur_gfi, map);
 
@@ -1300,7 +1317,9 @@ finalize_optimizer(Gif_Stream *gfs)
 	    || gfi->disposal == GIF_DISPOSAL_PREVIOUS)
 	&& gfi->delay && gfs->images[i-1]->delay) {
       Gif_UncompressImage(gfi);
-      if (gfi->img[0][0] == gfi->transparent) {
+      if (gfi->img[0][0] == gfi->transparent
+	  && (gfs->images[i-1]->disposal == GIF_DISPOSAL_ASIS
+	      || gfs->images[i-1]->disposal == GIF_DISPOSAL_NONE)) {
 	gfs->images[i-1]->delay += gfi->delay;
 	Gif_DeleteImage(gfi);
 	memmove(&gfs->images[i], &gfs->images[i+1], sizeof(Gif_Image *) * (gfs->nimages - i - 1));
